package com.example.nto.config;

import com.example.nto.entity.Employee;
import com.example.nto.repository.EmployeeRepository;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;

import java.util.Optional;

@Configuration
public class SecurityConfig {

    private final EmployeeRepository employeeRepository;

    public SecurityConfig(EmployeeRepository employeeRepository) {
        this.employeeRepository = employeeRepository;
    }

    @Bean
    public BCryptPasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    @Bean
    public UserDetailsService userDetailsService() {
        return username -> {
            Optional<Employee> employee = Optional.ofNullable(employeeRepository.findByLogin(username));
            if (employee.isEmpty()) {
                throw new RuntimeException("User not found");
            }

            Employee emp = employee.get();
            return User.builder()
                    .username(emp.getLogin())
                    .password(emp.getPassword())
                    .roles(emp.getRole().toUpperCase())
                    .build();
        };
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                .authorizeRequests()
                .antMatchers("/api/auth").permitAll()
                .antMatchers("/api/workers").hasRole("ADMIN") // Только админ может смотреть всех сотрудников
                .antMatchers("/api/{login}/block").hasRole("ADMIN") // Только админ может блокировать
                .anyRequest().authenticated() // Остальные запросы доступны всем авторизованным пользователям
                .and()
                .httpBasic()
                .and()
                .csrf().disable();

        return http.build();
    }
}