package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.service.EmployeeService;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RestController
@RequestMapping("/api")
public class EmployeeController {

    private final EmployeeService employeeService;

    public EmployeeController(EmployeeService employeeService) {
        this.employeeService = employeeService;
    }

    // Аутентификация
    @GetMapping("/auth")
    public ResponseEntity<String> authenticate(@RequestHeader("Authorization") String authHeader) {
        String[] credentials = new String(java.util.Base64.getDecoder().decode(authHeader.substring(6))).split(":");
        String login = credentials[0];
        String password = credentials.length > 1 ? credentials[1] : "";

        if (employeeService.authenticate(login, password)) {
            return ResponseEntity.ok("Authenticated successfully");
        }
        return ResponseEntity.status(401).body("Invalid login or password");
    }

    // Получение информации о пользователе
    @PreAuthorize("isAuthenticated()")
    @GetMapping("/{login}/info")
    public ResponseEntity<?> getUserInfo(@PathVariable String login, Authentication authentication) {
        Optional<Employee> employee = employeeService.findByLogin(login);

        if (employee.isPresent()) {
            Employee requestedUser = employee.get();
            String currentUserLogin = authentication.getName(); // Логин текущего пользователя
            boolean isAdmin = authentication.getAuthorities().stream()
                    .anyMatch(grantedAuthority -> grantedAuthority.getAuthority().equals("ROLE_ADMIN"));

            // Обычный пользователь может смотреть только свои данные, админ — любые
            if (isAdmin || currentUserLogin.equals(requestedUser.getLogin())) {
                return ResponseEntity.ok(requestedUser);
            } else {
                return ResponseEntity.status(403).body("Access denied");
            }
        }
        return ResponseEntity.status(404).body("User not found");
    }

    // не тз не трогаем!!!!
    @PreAuthorize("hasRole('ADMIN')")
    @GetMapping("/workers")
    public ResponseEntity<?> getAllWorkers() {
        return ResponseEntity.ok(employeeService.findAll());
    }

    // Блокировка/разблокировка пользователя (только для администратора)
    @PreAuthorize("hasRole('ADMIN')")
    @PatchMapping("/{login}/block")
    public ResponseEntity<String> toggleBlockStatus(@PathVariable String login) {
        employeeService.toggleBlockStatus(login);
        return ResponseEntity.ok("User " + login + " status changed");
    }

    // Открытие двери, если сотрудник не заблокирован
    @PreAuthorize("isAuthenticated()")
    @PatchMapping("/{login}/open")
    public ResponseEntity<String> openDoor(@PathVariable String login, @RequestBody Long code) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent() && !employee.get().isBlocked() && employeeService.validateCode(login, code)) {
            return ResponseEntity.ok("Door opened");
        }
        return ResponseEntity.status(403).body("Access denied");
    }
}